## soundcard.py - Contains the UI code needed for system-config-soundcard
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import string
import gtk
import gobject
import string
import sys
import os
import time
sys.path.append('/usr/share/system-config-soundcard/')
import soundcardBackend

##
## I18N
## 
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("system-config-soundcard")

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-config-soundcard/pixmaps/system-config-soundcard.png")
except:
    pass

class childWindow:
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 64
    moduleName = _("Soundcard Detection")
    commentTag = _("Auto-detect and configure soundcard")

    def destroy(self, args):
        gtk.main_quit()
    
    def __init__(self, doDebug=None, backend=None):
        self.doDebug = doDebug

        self.default_card = ""
        if backend == None:
            self.soundcardBackend = soundcardBackend.soundcardBackend()
        else:
            self.soundcardBackend = backend
        self.cardList = self.soundcardBackend.probeCards()

        if self.doDebug:
            print "Kudzu detected %d cards" % len(self.cardList)

        self.toplevel = gtk.VBox()
        self.icon_box = gtk.HBox()
        self.title = gtk.Label(_("Sound Card Selection"))
        self.title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse ("white"))

        #Create an internal box that will contain a list if there's multiple cards
        #Or just a box if there's only one.
        self.box = gtk.VBox(gtk.FALSE, 8)
        
        self.notebook = gtk.Notebook()

        if self.cardList == []:
            #There are no cards available
            self.label = gtk.Label(_("No soundcards were detected."))
            label = gtk.Label("")
            
        else:
            #Kudzu has found some cards
            for card in self.cardList:
                self.device, self.module, self.maker, self.model = self.soundcardBackend.getData(card)
                self.default_card = self.model

                self.table = gtk.Table(2, 3)
                self.table.set_border_width(6)
                self.table.set_col_spacings(4)
                self.table.set_row_spacings(4)

                label = gtk.Label(_(" %s" % (self.maker)))

                self.notebook.append_page(self.table, label)
                self.maker_label = gtk.Label("")
                self.maker_label.set_markup("<span><b>"
                           "%s</b></span>" % _("Vendor:"))
                
                self.maker_label.set_alignment(0.0, 0.5)
                self.maker_label2 = gtk.Label(self.maker)
                self.maker_label2.set_alignment(0.0, 0.5)

                self.model_label = gtk.Label("")
                self.model_label.set_markup("<span><b>"
                           "%s</b></span>" % _("Model:"))
                self.model_label.set_alignment(0.0, 0.5)

                self.model_label2 = gtk.Label(self.model)
                self.model_label2.set_alignment(0.0, 0.5)

                self.module_label = gtk.Label("")
                self.module_label.set_markup("<span><b>"
                           "%s</b></span>" % _("Module:"))
                self.module_label.set_alignment(0.0, 0.5)

                self.module_label2 = gtk.Label(_(self.module))
                self.module_label2.set_alignment(0.0, 0.5)

                self.test_button = gtk.Button(_("Play test sound"))
                self.test_button.connect("clicked", self.testButtonClicked)

                align = gtk.Alignment(1.0, 0.5)
                align.add(self.test_button)

                self.table.attach(self.maker_label, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
                self.table.attach(self.maker_label2, 1, 2, 0, 1, gtk.FILL, gtk.FILL)
                self.table.attach(self.model_label, 0, 1, 1, 2, gtk.FILL, gtk.FILL)
                self.table.attach(self.model_label2, 1, 2, 1, 2, gtk.FILL, gtk.FILL)
                self.table.attach(self.module_label, 0, 1, 2, 3, gtk.FILL, gtk.FILL)
                self.table.attach(self.module_label2, 1, 2, 2, 3, gtk.FILL, gtk.FILL)
                self.table.attach(align, 1, 2, 3, 4)


            if len(self.cardList) == 1:
                #There is only one card, so hide the tab and border
                self.notebook.set_show_tabs(gtk.FALSE)
                self.notebook.set_show_border(gtk.FALSE)
                self.label = gtk.Label(_("The following audio device was detected."))
                self.box.pack_start(self.notebook)            


            else:
                #There is more than one card, so show the primary device option menu
                self.label = gtk.Label(_("The following audio devices were detected."))

                #Create a menu
                deviceBox = gtk.HBox(gtk.FALSE, 8)
                self.primaryDeviceOptionMenu = gtk.OptionMenu()
                self.primaryDeviceMenu = gtk.Menu()

                for card in self.cardList:
                    self.device, self.module, self.maker, self.model = self.soundcardBackend.getData(card)
                    item = gtk.MenuItem(self.model)
                    self.primaryDeviceMenu.append(item)

                self.primaryDeviceMenu.set_active(self.cardList.index(self.soundcardBackend.getDefaultCard()))

                self.primaryDeviceOptionMenu.set_menu(self.primaryDeviceMenu)


                self.box.pack_start(self.notebook)
                deviceBox.pack_start(gtk.Label(_("Primary audio device:")), gtk.FALSE)
                deviceBox.pack_start(self.primaryDeviceOptionMenu, gtk.TRUE)
                self.box.pack_start(deviceBox)

		self.primaryDeviceOptionMenu.connect("changed", self.changeDefault)

        #Add icon to the top frame
        p = None
        try:
            p = gtk.gdk.pixbuf_new_from_file("../pixmaps/system-config-soundcard.png")
        except:
            try:
                p = gtk.gdk.pixbuf_new_from_file("/usr/share/system-config-soundcard/pixmaps/system-config-soundcard.png")
            except:
                pass

        if p:
            self.icon = gtk.Image()
            self.icon.set_from_pixbuf(p)

    def closeClicked(self, *args):
        self.apply()
        gtk.main_quit()

    def changeDefault(self, *args):
	i = self.primaryDeviceOptionMenu.get_history()
	self.soundcardBackend.setDefault(i)

    def apply(self, *args):
        if self.doDebug:
            return

        pageNum = self.notebook.get_current_page()
        page = self.notebook.get_nth_page(pageNum)

        if page:
            module = page.get_children()[1].get_text()

            selectedItem = self.getSelectedCard()
##             if selectedItem != None:
##                 self.writeUnloadReload(selectedItem, module)
        
        self.mainWindow.destroy()

    def testButtonClicked(self, button):
        pageNum = self.notebook.get_current_page()
        page = self.notebook.get_nth_page(pageNum)
        module = page.get_children()[1].get_text()
        index = self.notebook.get_current_page()

        selectedItem = self.getSelectedCard()

        if selectedItem != None:
#            self.writeUnloadReload(selectedItem, module)
            self.soundcardBackend.playTestSound(module, index)

            text = (_("Did you hear the sample sound?"))
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,text)

            dlg.set_title("")
            dlg.set_default_size(100, 100)
            dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
            dlg.set_border_width(2)
            dlg.set_modal(gtk.TRUE)
            rc = dlg.run()
            dlg.destroy()

            if rc == gtk.RESPONSE_NO:
                #Oh well, they couldn't hear the sound.  Notify the user and then quit.  We did our best
                self.showErrorDialog(_("Automatic detection of the sound card did not work.  " \
                                       "Audio will not be available on the system.  Please click " \
                                       "OK to continue."))
                try:
                    self.mainWindow.destroy()
                except:
                    #We must be in firstboot mode
                    pass

    def getSelectedCard(self):
        if len(self.cardList) == 1:
            #There is one sound card present.  This is the common case
            selectedCard = self.soundcardBackend.getData(self.cardList[0])
            device, module, maker, model = selectedCard
            self.default_card = model
            return 0

        elif len(self.cardList) > 1:
            #There are more than one soundcards.  Figure out which is supposed the be the default
            selectedItem = self.primaryDeviceOptionMenu.get_history()
            selectedCard = self.soundcardBackend.getData(self.cardList[selectedItem])
            device, module, maker, model = selectedCard
            self.default_card = model
            return selectedItem

        else:
            #There are no soundcards.  Do nothing
            return None

    def writeUnloadReload(self, selectedItem, module):
        self.soundcardBackend.writeFile(selectedItem)
        self.soundcardBackend.unloadModules()
        result = self.soundcardBackend.loadModules()

        if result != 0:
            #Loading the module failed.  Notify the user that things are broken
            self.showErrorDialog(_("The %s driver could not be loaded." % module))
        else:
            self.soundcardBackend.setVolume()

    def launch(self):
        self.mainVBox = gtk.VBox()
        self.internalVBox = gtk.VBox(gtk.FALSE, 10)
        self.internalVBox.set_border_width(10)

        eventBox = gtk.EventBox()
        eventBox.add(self.icon_box)
        eventBox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse ("#cc0000"))

        self.icon_box.pack_start(self.icon, gtk.FALSE, gtk.TRUE)        
        self.icon_box.pack_start(self.title)
        self.mainVBox.pack_start(eventBox, gtk.FALSE)
        self.msg_label.set_size_request(500, -1)
        self.internalVBox.pack_start(self.msg_label, gtk.FALSE)
        self.mainVBox.pack_start(self.internalVBox, gtk.TRUE)
        return self.mainVBox, eventBox

    def stand_alone(self):
        self.mainWindow = gtk.Dialog()
        self.mainWindow.vbox.set_spacing(5)
        self.mainWindow.connect("destroy", self.destroy)
        self.mainWindow.set_title(_("Audio Devices"))
        self.mainWindow.set_border_width(10)
        self.mainWindow.set_position(gtk.WIN_POS_CENTER)
        self.mainWindow.set_icon(iconPixbuf)
        self.okButton = self.mainWindow.add_button('gtk-close', 0)
        self.okButton.connect("clicked", self.closeClicked)

        self.toplevel.set_spacing(5)
        self.icon_box.pack_start(self.icon, gtk.FALSE)
        self.icon_box.pack_start(self.label, gtk.FALSE)        
        self.toplevel.pack_start(self.icon_box, gtk.FALSE)

        self.toplevel.pack_start(self.box, gtk.FALSE)

        #Remove the hsep from the dialog.  It's ugly
        hsep = self.mainWindow.get_children()[0].get_children()[0]
        self.mainWindow.get_children()[0].remove(hsep)
        self.mainWindow.vbox.pack_start(self.toplevel)
        self.mainWindow.show_all()
        gtk.main()

    #The getData function can return the name of the setting being changed
    #This is useful for the hardware screen in firstboot
    def getData(self):
        return self.default_card
        
    def showErrorDialog(self, text):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, text)

        dlg.set_title(_("Error"))
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        dlg.set_border_width(2)
        dlg.set_modal(gtk.TRUE)
        rc = dlg.run()
        dlg.destroy()
        return

    def getSoundcardBox(self):
        return self.box
